#!/usr/bin/python
#
# Copyright 2014 Canonical Ltd.  All rights reserved
# Author: Chris Stratford <chris.stratford@canonical.com>

import sys
import os
import os.path
import subprocess
import json
import base64
import shutil
sys.path.insert(0, os.path.join(os.environ['CHARM_DIR'], 'lib'))
from charmhelpers.core.hookenv import Hooks, log, charm_dir
from charmhelpers.core.host import mkdir, service_start, service_stop, service_restart
from charmhelpers.fetch import apt_install, apt_update, add_source
from Cheetah.Template import Template
from Config import Config
#from nrpe import update_nrpe_checks
from charmhelpers.contrib.charmsupport import nrpe

hooks = Hooks()
conf = Config()

def jujuHeader():
    header = ("#-------------------------------------------------#\n"
              "# This file is Juju managed - do not edit by hand #\n"
              "#-------------------------------------------------#\n")
    return header

def installPackages():
    if conf.packageName():
        pkgFile = os.path.join(charm_dir(), "files", conf.packageName())
        if os.path.exists(pkgFile):
            if subprocess.call(["dpkg", "-i", pkgFile]) != 0:
                log("CHARM: Error installing logstash-forwarder package")
                sys.exit(1)
        else:
            if conf.aptRepository() and conf.aptRepository_key():
                add_source(conf.aptRepository(), conf.aptRepositoryKey())
                apt_update()
            apt_install(conf.packageName(), options=['--force-yes'])


def replaceInitScript():
    src = os.path.join(charm_dir(), "files", "init.sh")
    dest = "/etc/init.d/logstash-forwarder"
    shutil.copyfile(src, dest)
    os.chmod(dest, 0755)


def writeEtcDefault():
    tmplData = {}
    tmplData["config_file"] = conf.configFile()
    templateFile = os.path.join(charm_dir(), "templates", "etcdefault.tmpl")
    t = Template(file=templateFile, searchList=tmplData)
    with open("/etc/default/logstash-forwarder", "w") as f:
        os.chmod("/etc/default/logstash-forwarder", 0644)
        f.write(jujuHeader())
        f.write(str(t))


def writeSSL():
    if not os.path.exists(conf.configDir()):
        mkdir(conf.configDir())
    if conf.sslCert():
        with open(conf.sslCertFile(), "w") as f:
            os.chmod(conf.sslCertFile(), 0644)
            f.write(base64.b64decode(conf.sslCert()))
    if conf.sslKey():
        with open(conf.sslKeyFile(), "w") as f:
            os.chmod(conf.sslKeyFile(), 0600)
            f.write(base64.b64decode(conf.sslKey()))
    if conf.sslCACert():
        with open(conf.sslCACertFile(), "w") as f:
            os.chmod(conf.sslCACertFile(), 0644)
            f.write(base64.b64decode(conf.sslCACert()))


def writeConfig():
    config = {}
    servers = []
    files = []
    if not os.path.exists(conf.configDir()):
        mkdir(conf.configDir())
    serverPort = conf.serverPort()
    for server in conf.servers():
        servers.append(server + ":" + str(serverPort))
    config["network"] = {}
    config["network"]["servers"] = servers
    if conf.sslCert():
        config["network"]["ssl certificate"] = conf.sslCertFile()
    if conf.sslKey():
        config["network"]["ssl key"] = conf.sslKeyFile()
    if conf.sslCACert():
        config["network"]["ssl ca"] = conf.sslCACertFile()
    if conf.timeout():
        config["network"]["timeout"] = conf.timeout()

    fileSpec = conf.files()
    if fileSpec is not None:
        for logType in fileSpec:
            files.append({"paths": fileSpec[logType], "fields": {"type": logType}})
        config["files"] = files

    with open(conf.configFile(), "w") as f:
        f.write(json.dumps(config, sort_keys=True, indent=4))


def writeEtcHosts():
    log("CHARM: updating /etc/hosts with logstash servers name mapping")
    name_map = conf.serversNameMap()
    servers = conf.servers()
    if not name_map:
        return
    new_hosts = []
    for server in servers:
        ipaddr = name_map[server]
        new_hosts.append('{} {}'.format(ipaddr.strip(), server.strip()))
    # update /etc/hosts
    with open('/etc/hosts') as f:
        current_hosts = f.readlines()
    section_start = "### logstash-forwarder start ###\n"
    section_end = "### logstash-forwarder end ###\n"
    with open('/tmp/new_hosts', 'w') as f:
        ignore_lines = False
        for host_line in current_hosts:
            if host_line == section_start:
                ignore_lines = True
            elif host_line == section_end:
                ignore_lines = False
            elif not ignore_lines:
                f.write(host_line)
        # now write a new section for our hosts
        f.write("### logstash-forwarder start ###\n")
        for new_host_line in new_hosts:
            f.write("{}\n".format(new_host_line))
        f.write("### logstash-forwarder end ###\n")
    os.rename('/tmp/new_hosts', '/etc/hosts')



@hooks.hook('nrpe-external-master-relation-changed')
def update_nrpe_checks():
    nrpe_compat = nrpe.NRPE()
    conf = nrpe_compat.config
    check_procs_params = conf.get('nagios_check_procs_params')
    if check_procs_params:
        nrpe_compat.add_check(
            shortname='logstashforwarder_process',
            description='Check logstash-forwarder process running',
            check_cmd='check_procs %s' % check_procs_params
        )
    nrpe_compat.add_check(
        shortname='logstashforwarder_sending',
        description='Check logstash-forwarder is sending by tailing logfile',
        check_cmd = "/usr/local/lib/nagios/plugins/check-logstashforwarder-sending.sh"
        )
    nrpe_compat.write()
    copy_check_files()


def copy_check_files():
    script_dir="/usr/local/lib/nagios/plugins"
    mkdir(script_dir)
    for script in ["check-logstashforwarder-sending.sh"]:
        src = os.path.join(charm_dir(), "files", script)
        dest = os.path.join(script_dir, script)
        shutil.copyfile(src, dest)
        os.chmod(dest, 0755)


@hooks.hook("install")
def install():
    log("CHARM: Installing {}".format(conf.appName()))
    installPackages()
    replaceInitScript()


@hooks.hook("upgrade-charm")
def upgrade_charm():
    log("CHARM: Upgrading {}".format(conf.appName()))


@hooks.hook("config-changed")
def config_changed():
    log("CHARM: Configuring {}".format(conf.appName()))
    installPackages()
    replaceInitScript()
    writeEtcDefault()
    writeSSL()
    writeEtcHosts()
    writeConfig()
    service_restart("logstash-forwarder")
    update_nrpe_checks()


@hooks.hook("start")
def start():
    log("CHARM: Starting {}".format(conf.appName()))
    service_start("logstash-forwarder")


@hooks.hook("stop")
def stop():
    log("CHARM: Stopping {}".format(conf.appName()))
    service_stop("logstash-forwarder")


if __name__ == "__main__":
    hooks.execute(sys.argv)
